@@ -359,6 +359,7 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 
 		add_lang_action (totem, action_group, ui_id, path, prefix,
 				 action_data, i, num + 1, &group);
+		g_free (action_data);
  		i++;
 	}
 
@@ -545,29 +546,11 @@ on_recent_file_item_activated (GtkAction *action,
 {
 	GtkRecentInfo *recent_info;
 	const gchar *uri;
-	gboolean playlist_changed;
-	guint end;
 
 	recent_info = g_object_get_data (G_OBJECT (action), "recent-info");
 	uri = gtk_recent_info_get_uri (recent_info);
 
-	totem_signal_block_by_data (totem->playlist, totem);
-
-	end = totem_playlist_get_last (totem->playlist);
-	playlist_changed = totem_playlist_add_mrl_with_cursor (totem->playlist, uri, NULL);
-	gtk_recent_manager_add_item (totem->recent_manager, uri);
-
-	totem_signal_unblock_by_data (totem->playlist, totem);
-
-	if (playlist_changed)
-	{
-		char *mrl;
-
-		totem_playlist_set_current (totem->playlist, end + 1);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
-	}
+	totem_add_to_playlist_and_play (totem, uri, NULL, TRUE);
 }
 
 static gint
@@ -1136,7 +1119,7 @@ about_action_callback (GtkAction *action, Totem *totem)
 
 	gtk_show_about_dialog (GTK_WINDOW (totem->win),
 				     "version", VERSION,
-				     "copyright", _("Copyright \xc2\xa9 2002-2006 Bastien Nocera"),
+				     "copyright", _("Copyright \xc2\xa9 2002-2007 Bastien Nocera"),
 				     "comments", description,
 				     "authors", authors,
 				     "documenters", documenters,
@@ -1270,7 +1253,7 @@ void
 clear_playlist_action_callback (GtkAction *action, Totem *totem)
 {
 	totem_playlist_clear (totem->playlist);
-	totem_action_set_mrl (totem, NULL);
+	totem_action_set_mrl (totem, NULL, NULL);
 }
 
 static const GtkActionEntry seek_entries_ltr[] = {
