@@ -65,7 +65,7 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
-#include "totem-disc.h"
+#include "plparse/totem-disc.h"
 
 #include "debug.h"
 
@@ -92,7 +92,7 @@
 
 static const GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0 },
-	{ "_NETSCAPE_URL", 0, 1 },
+	{ "_NETSCAPE_URL", 0, 1 }
 };
 
 static gboolean totem_action_open_files (Totem *totem, char **list);
@@ -157,11 +157,10 @@ totem_action_save_size (Totem *totem)
 }
 
 static void
-totem_action_save_state (Totem *totem)
+totem_action_save_state (Totem *totem, const char *page_id)
 {
 	GKeyFile *keyfile;
 	char *contents, *filename;
-	const char *page_id;
 
 	if (totem->win == NULL)
 		return;
@@ -181,7 +180,6 @@ totem_action_save_state (Totem *totem)
 	g_key_file_set_integer (keyfile, "State",
 			"sidebar_w", totem->sidebar_w);
 
-	page_id = totem_sidebar_get_current_page (totem);
 	g_key_file_set_string (keyfile, "State",
 			"sidebar_page", page_id);
 
@@ -198,6 +196,7 @@ void
 totem_action_exit (Totem *totem)
 {
 	GdkDisplay *display = NULL;
+	char *page_id;
 
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
@@ -213,6 +212,9 @@ totem_action_exit (Totem *totem)
 	if (totem->prefs != NULL)
 		gtk_widget_hide (totem->prefs);
 
+	/* Save the page ID before we close the plugins, otherwise
+	 * we'll never save it properly */
+	page_id = totem_sidebar_get_current_page (totem);
 	totem_object_plugins_shutdown ();
 
 	if (display != NULL)
@@ -222,7 +224,7 @@ totem_action_exit (Totem *totem)
 		int vol;
 
 		vol = bacon_video_widget_get_volume (totem->bvw) * 100.0 + 0.5;
-		//FIXME move the volume to the static file?
+		/* FIXME move the volume to the static file? */
 		gconf_client_set_int (totem->gc,
 				GCONF_PREFIX"/volume",
 				CLAMP (vol, 0, 100),
@@ -230,8 +232,10 @@ totem_action_exit (Totem *totem)
 		totem_action_save_size (totem);
 	}
 
-	bacon_message_connection_free (totem->conn);
-	totem_action_save_state (totem);
+	if (totem->conn != NULL)
+		bacon_message_connection_free (totem->conn);
+	totem_action_save_state (totem, page_id);
+	g_free (page_id);
 
 	totem_sublang_exit (totem);
 	totem_destroy_file_filters ();
@@ -331,32 +335,20 @@ play_pause_set_label (Totem *totem, TotemStates state)
 void
 totem_action_eject (Totem *totem)
 {
-	GError *err = NULL;
-	char *cmd, *prefix;
-	const char *needle;
-	char *device;
+	GnomeVFSVolume *volume;
 
-	if (totem->mrl == NULL)
+	volume = totem_get_volume_for_media (totem->mrl);
+	if (volume == NULL)
 		return;
 
-	needle = strchr (totem->mrl, ':');
-	g_assert (needle != NULL);
-	/* we want the ':' as well */
-	prefix = g_strndup (totem->mrl, needle - totem->mrl + 1);
-	totem_playlist_clear_with_prefix (totem->playlist, prefix);
-	g_free (prefix);
+	g_free (totem->mrl);
+	totem->mrl = NULL;
+	bacon_video_widget_close (totem->bvw);
+	totem_file_closed (totem);
 
-	g_object_get (G_OBJECT (totem->bvw),
-			"mediadev", &device, NULL);
-	cmd = g_strdup_printf ("eject %s", device);
-	g_free (device);
-
-	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, &err) == FALSE)
-	{
-		totem_action_error (_("Totem could not eject the optical media."), err->message, totem);
-		g_error_free (err);
-	}
-	g_free (cmd);
+	/* the volume monitoring will take care of removing the items */
+	gnome_vfs_volume_eject (volume, NULL, NULL);
+	gnome_vfs_volume_unref (volume);
 }
 
 void
@@ -431,9 +423,9 @@ totem_action_seek (Totem *totem, double pos)
 }
 
 void
-totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
+totem_action_set_mrl_and_play (Totem *totem, const char *mrl, const char *subtitle)
 {
-	if (totem_action_set_mrl (totem, mrl) != FALSE)
+	if (totem_action_set_mrl (totem, mrl, subtitle) != FALSE)
 		totem_action_play (totem);
 }
 
@@ -461,11 +453,12 @@ totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 	g_slist_free (filenames);
 
 	if (play != FALSE) {
-		char *mrl;
+		char *mrl, *subtitle;
 
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
 
 	return TRUE;
@@ -533,12 +526,20 @@ totem_action_load_media_device (Totem *totem, const char *device)
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
-		case MEDIA_TYPE_CDDA:
-			retval = totem_action_load_media (totem, type, device_path);
-			if (retval == FALSE) {
-				totem_action_set_mrl_and_play (totem, NULL);
+			{
+				const char *filenames[2];
+
+				filenames[0] = url;
+				filenames[1] = NULL;
+
+				retval = totem_action_open_files (totem, (char **) filenames);
 			}
 			break;
+		case MEDIA_TYPE_CDDA:
+			totem_action_error (_("Totem does not support playback of Audio CDs"),
+					    _("Please consider using a music player or a CD extractor to play this CD"),
+					    totem);
+			retval = FALSE;
 		default:
 			g_assert_not_reached ();
 	}
@@ -555,10 +556,10 @@ totem_action_play_media_device (Totem *totem, const char *device)
 	char *mrl;
 
 	if (totem_action_load_media_device (totem, device) != FALSE) {
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, NULL);
+		totem_action_set_mrl_and_play (totem, mrl, NULL);
 		g_free (mrl);
-	} 
+	}
 }
 
 void
@@ -567,8 +568,8 @@ totem_action_play_media (Totem *totem, TotemDiscMediaType type, const char *devi
 	char *mrl;
 
 	if (totem_action_load_media (totem, type, device) != FALSE) {
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, NULL);
+		totem_action_set_mrl_and_play (totem, mrl, NULL);
 		g_free (mrl);
 	}
 }
@@ -585,16 +586,17 @@ totem_action_play_pause (Totem *totem)
 {
 	if (totem->mrl == NULL)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		/* Try to pull an mrl from the playlist */
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 		if (mrl == NULL) {
 			play_pause_set_label (totem, STATE_STOPPED);
 			return;
 		} else {
-			totem_action_set_mrl_and_play (totem, mrl);
+			totem_action_set_mrl_and_play (totem, mrl, subtitle);
 			g_free (mrl);
+			g_free (subtitle);
 			return;
 		}
 	}
@@ -722,16 +724,17 @@ totem_open_location_response_cb (GtkDialog *dialog, gint response, Totem *totem)
 
 	if (uri != NULL)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 		const char *filenames[2];
 
 		filenames[0] = uri;
 		filenames[1] = NULL;
 		totem_action_open_files (totem, (char **) filenames);
 
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
  	g_free (uri);
 
@@ -845,7 +848,7 @@ update_mrl_label (Totem *totem, const char *name)
 		/* Get the length of the stream */
 		time = bacon_video_widget_get_stream_length (totem->bvw);
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
-				(totem->statusbar), 0, time / 1000);
+						     (totem->statusbar), 0, time / 1000);
 
 		g_object_notify (G_OBJECT (totem), "stream-length");
 
@@ -871,8 +874,10 @@ update_mrl_label (Totem *totem, const char *name)
 }
 
 gboolean
-totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
-		gboolean warn)
+totem_action_set_mrl_with_warning (Totem *totem,
+				   const char *mrl, 
+				   const char *subtitle,
+				   gboolean warn)
 {
 	gboolean retval = TRUE;
 
@@ -918,15 +923,18 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 	} else {
 		gboolean caps;
 		gdouble volume;
-		char *subtitle_uri;
+		char *autoload_sub = NULL;
 		GError *err = NULL;
 
 		bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 
-		subtitle_uri = totem_uri_get_subtitle_uri (mrl);
+		if (subtitle == NULL && totem->autoload_subs != FALSE)
+			autoload_sub = totem_uri_get_subtitle_uri (mrl);
+
 		totem_gdk_window_set_waiting_cursor (totem->win->window);
-		retval = bacon_video_widget_open_with_subtitle (totem->bvw,
-				mrl, subtitle_uri, &err);
+		retval = bacon_video_widget_open_with_subtitle (totem->bvw, mrl,
+								subtitle ? subtitle : autoload_sub, &err);
+		g_free (autoload_sub);
 		gdk_window_set_cursor (totem->win->window, NULL);
 		totem->mrl = g_strdup (mrl);
 
@@ -985,9 +993,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 }
 
 gboolean
-totem_action_set_mrl (Totem *totem, const char *mrl)
+totem_action_set_mrl (Totem *totem, const char *mrl, const char *subtitle)
 {
-	return totem_action_set_mrl_with_warning (totem, mrl, TRUE);
+	return totem_action_set_mrl_with_warning (totem, mrl, subtitle, TRUE);
 }
 
 static gboolean
@@ -1021,11 +1029,13 @@ totem_action_direction (Totem *totem, TotemPlaylistDirection dir)
 			|| bacon_video_widget_is_seekable (totem->bvw) == FALSE
 			|| totem_time_within_seconds (totem) != FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		totem_playlist_set_direction (totem->playlist, dir);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
+
+		g_free (subtitle);
 		g_free (mrl);
 	} else {
 		totem_action_seek (totem, 0);
@@ -1288,14 +1298,15 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 	/* ... and reconnect because we're nice people */
 	if (cleared != FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		g_signal_connect (G_OBJECT (totem->playlist),
 				"changed", G_CALLBACK (playlist_changed_cb),
 				totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
 
 	return TRUE;
@@ -1371,8 +1382,8 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
 	gchar *old_mrl, *new_mrl;
 
-	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist));
-	new_mrl = totem_resolve_relative_link (old_mrl, mrl);
+	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist), NULL);
+	new_mrl = totem_pl_parser_resolve_url (old_mrl, mrl);
 	g_free (old_mrl);
 
 	bacon_video_widget_close (totem->bvw);
@@ -1385,14 +1396,14 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 	g_free (new_mrl);
 }
 
-/* This is only called when we are playing a DVD */
+/* This is only called when we are playing a DVD or a radio */
 static void
 on_title_change_event (BaconVideoWidget *bvw, const char *string, Totem *totem)
 {
 	update_mrl_label (totem, string);
 	update_buttons (totem);
 	totem_playlist_set_title (TOTEM_PLAYLIST (totem->playlist),
-			string, TRUE);
+				  string, TRUE);
 }
 
 static void
@@ -1441,7 +1452,7 @@ static void
 on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 {
         char *name = NULL;
-
+	
 	totem_metadata_updated (totem, NULL, NULL, NULL);
 
 	name = totem_get_nice_name_for_stream (totem);
@@ -1451,8 +1462,11 @@ on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 			(TOTEM_PLAYLIST (totem->playlist), name, FALSE);
 		g_free (name);
 	}
-	on_playlist_change_name 	 
-		(TOTEM_PLAYLIST (totem->playlist), totem);
+	
+	totem_action_set_sensitivity ("take-screenshot",
+				      bacon_video_widget_can_get_frames (bvw, NULL));
+	
+	on_playlist_change_name (TOTEM_PLAYLIST (totem->playlist), totem);
 }
 
 static void
@@ -1479,6 +1493,8 @@ on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 static void
 update_seekable (Totem *totem)
 {
+	GtkAction *action;
+	GtkActionGroup *action_group;
 	gboolean seekable;
 
 	seekable = bacon_video_widget_is_seekable (totem->bvw);
@@ -1493,8 +1509,17 @@ update_seekable (Totem *totem)
 
 	totem_fullscreen_set_seekable (totem->fs, seekable);
 
-	totem_action_set_sensitivity ("skip-forward", seekable);
-	totem_action_set_sensitivity ("skip-backwards", seekable);
+	/* FIXME: We can use this code again once bug #457631 is fixed and
+	 * skip-* are back in the main action group. */
+	/*totem_action_set_sensitivity ("skip-forward", seekable);
+	totem_action_set_sensitivity ("skip-backwards", seekable);*/
+	action_group = GTK_ACTION_GROUP (gtk_builder_get_object (totem->xml, "skip-action-group"));
+
+	action = gtk_action_group_get_action (action_group, "skip-forward");
+	gtk_action_set_sensitive (action, seekable);
+
+	action = gtk_action_group_get_action (action_group, "skip-backwards");
+	gtk_action_set_sensitive (action, seekable);
 
 	g_object_notify (G_OBJECT (totem), "seekable");
 }
@@ -1503,7 +1528,7 @@ static void
 update_current_time (BaconVideoWidget *bvw,
 		gint64 current_time,
 		gint64 stream_length,
-		float current_position,
+		double current_position,
 		gboolean seekable, Totem *totem)
 {
 	if (totem->seek_lock == FALSE)
@@ -1686,8 +1711,6 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				|| strstr (filename, "://") != NULL
 				|| g_str_has_prefix (filename, "dvd:") != FALSE
 				|| g_str_has_prefix (filename, "vcd:") != FALSE
-				|| g_str_has_prefix (filename, "cdda:") != FALSE
-				|| g_str_has_prefix (filename, "cd:") != FALSE
 				|| g_str_has_prefix (filename, "dvb:") != FALSE)
 		{
 			if (cleared == FALSE)
@@ -1707,9 +1730,6 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 			if (totem_is_block_device (filename) != FALSE) {
 				totem_action_load_media_device (totem, data);
 				changed = TRUE;
-			} else if (g_str_has_prefix (filename, "cdda:/") != FALSE) {
-				totem_playlist_add_mrl (totem->playlist, data, NULL);
-				changed = TRUE;
 			} else if (g_str_has_prefix (filename, "dvb:/") != FALSE) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
 				changed = TRUE;
@@ -1850,12 +1870,13 @@ totem_action_next_angle (Totem *totem)
 void
 totem_action_set_playlist_index (Totem *totem, guint index)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	totem_playlist_set_current (totem->playlist, index);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
-	totem_action_set_mrl_and_play (totem, mrl);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
 	g_free (mrl);
+	g_free (subtitle);
 }
 
 void
@@ -1908,22 +1929,17 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		if (url == NULL) {
 			bacon_video_widget_close (totem->bvw);
 			totem_file_closed (totem);
-			totem_action_set_mrl_and_play (totem, NULL);
+			totem_action_set_mrl (totem, NULL, NULL);
 			break;
 		}
 		if (strcmp (url, "dvd:") == 0) {
-			//FIXME b0rked
+			/* FIXME b0rked */
 			totem_action_play_media (totem, MEDIA_TYPE_DVD, NULL);
 		} else if (strcmp (url, "vcd:") == 0) {
-			//FIXME b0rked
+			/* FIXME b0rked */
 			totem_action_play_media (totem, MEDIA_TYPE_VCD, NULL);
-		} else if (g_str_has_prefix (url, "cd:") != FALSE) {
-			//FIXME b0rked
-			totem_action_play_media (totem, MEDIA_TYPE_CDDA, NULL);
 		} else if (g_str_has_prefix (url, "dvb:") != FALSE) {
 			totem_action_play_media (totem, MEDIA_TYPE_DVB, NULL);
-		} else if (g_str_has_prefix (url, "cdda:/") != FALSE) {
-			totem_playlist_add_mrl_with_cursor (totem->playlist, url, NULL);
 		} else if (totem_playlist_add_mrl_with_cursor (totem->playlist, url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
 		}
@@ -2004,7 +2020,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		totem_action_eject (totem);
 		break;
 	case TOTEM_REMOTE_COMMAND_PLAY_DVD:
-		// TODO - how to see if can, and play the DVD (like the menu item)
+		/* TODO - how to see if can, and play the DVD (like the menu item) */
 		break;
 	case TOTEM_REMOTE_COMMAND_MUTE:
 		totem_action_volume_relative (totem, -1.0);
@@ -2066,27 +2082,19 @@ gboolean totem_action_remote_get_setting (Totem *totem,
 static void
 playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	update_buttons (totem);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 
 	if (mrl == NULL)
 		return;
 
-	//FIXME we shouldn't compare the 2 URLs
-	// http://bugzilla.gnome.org/show_bug.cgi?id=364311
-	if (totem->mrl == NULL
-			|| (totem->mrl != NULL && mrl != NULL
-			&& strcmp (totem->mrl, mrl) != 0))
-	{
-		totem_action_set_mrl_and_play (totem, mrl);
-	} else if (totem->mrl != NULL) {
-		//FIXME that's busted
-		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_PLAYING);
-	}
+	if (totem_playlist_get_playing (totem->playlist) == TOTEM_PLAYLIST_STATUS_NONE)
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 
 	g_free (mrl);
+	g_free (subtitle);
 }
 
 static void
@@ -2098,23 +2106,38 @@ item_activated_cb (GtkWidget *playlist, Totem *totem)
 static void
 current_removed_cb (GtkWidget *playlist, Totem *totem)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	/* Set play button status */
 	play_pause_set_label (totem, STATE_STOPPED);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 
-	if (mrl == NULL)
-	{
+	if (mrl == NULL) {
+		g_free (subtitle);
+		subtitle = NULL;
 		totem_playlist_set_at_start (totem->playlist);
 		update_buttons (totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 	} else {
 		update_buttons (totem);
 	}
 
-	totem_action_set_mrl_and_play (totem, mrl);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
 	g_free (mrl);
+	g_free (subtitle);
+}
+
+static void
+subtitle_changed_cb (GtkWidget *playlist, Totem *totem)
+{
+	char *mrl, *subtitle;
+
+	totem_action_stop (totem);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
+
+	g_free (mrl);
+	g_free (subtitle);
 }
 
 static void
@@ -2220,16 +2243,17 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 	if (totem_playlist_has_next_mrl (totem->playlist) == FALSE
 			&& totem_playlist_get_repeat (totem->playlist) == FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		/* Set play button status */
 		totem_playlist_set_at_start (totem->playlist);
 		update_buttons (totem);
 		totem_action_stop (totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_with_warning (totem, mrl, FALSE);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_with_warning (totem, mrl, subtitle, FALSE);
 		bacon_video_widget_pause (totem->bvw);
 		g_free (mrl);
+		g_free (subtitle);
 	} else {
 		totem_action_next (totem);
 	}
@@ -2276,14 +2300,7 @@ totem_action_handle_seek (Totem *totem, GdkEventKey *event, gboolean is_forward)
 static gboolean
 totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 {
-	gboolean retval = TRUE, playlist_focused = FALSE;
-	GtkWidget *focused;
-
-	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
-	if (focused != NULL && gtk_widget_is_ancestor
-			(focused, GTK_WIDGET (totem->playlist)) != FALSE) {
-		playlist_focused = TRUE;
-	}
+	gboolean retval = TRUE;
 
 	switch (event->keyval) {
 	case GDK_A:
@@ -2414,18 +2431,12 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			totem_action_fullscreen (totem, FALSE);
 		break;
 	case GDK_Left:
-		if (playlist_focused != FALSE)
-			return FALSE;
-
 		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
 			totem_action_handle_seek (totem, event, TRUE);
 		else
 			totem_action_handle_seek (totem, event, FALSE);
 		break;
 	case GDK_Right:
-		if (playlist_focused != FALSE)
-			return FALSE;
-
 		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
 			totem_action_handle_seek (totem, event, FALSE);
 		else
@@ -2438,13 +2449,9 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			retval = FALSE;
 		break;
 	case GDK_Up:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
 		break;
 	case GDK_Down:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_DOWN_OFFSET);
 		break;
 	case GDK_0:
@@ -2463,13 +2470,9 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		totem_action_set_scale_ratio (totem, 2);
 		break;
 	case GDK_Menu:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_menu_popup (totem, 0);
 		break;
 	case GDK_F10:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		if (!(event->state & GDK_SHIFT_MASK))
 			return FALSE;
 
@@ -2492,23 +2495,27 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 	case GDK_KP_Up:
 	case GDK_KP_8:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_UP);
+					      BVW_DVD_ROOT_MENU_UP);
 		break;
 	case GDK_KP_Down:
 	case GDK_KP_2:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_DOWN);
+					      BVW_DVD_ROOT_MENU_DOWN);
 		break;
 	case GDK_KP_Right:
 	case GDK_KP_6:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_RIGHT);
+					      BVW_DVD_ROOT_MENU_RIGHT);
 		break;
 	case GDK_KP_Left:
 	case GDK_KP_4:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_LEFT);
+					      BVW_DVD_ROOT_MENU_LEFT);
 		break;
+	case GDK_KP_Begin:
+	case GDK_KP_5:
+		bacon_video_widget_dvd_event (totem->bvw,
+					      BVW_DVD_ROOT_MENU_SELECT);
 	default:
 		retval = FALSE;
 	}
@@ -2540,6 +2547,14 @@ totem_action_handle_scroll (Totem *totem, GdkScrollDirection direction)
 int
 window_key_press_event_cb (GtkWidget *win, GdkEventKey *event, Totem *totem)
 {
+	GtkWidget *focused;
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->sidebar)) != FALSE) {
+		return FALSE;
+	}
+
 	/* Special case Eject, Open, Open URI and
 	 * seeking keyboard shortcuts */
 	if (event->state != 0
@@ -2609,7 +2624,8 @@ window_scroll_event_cb (GtkWidget *win, GdkEventScroll *event, Totem *totem)
 static void
 update_media_menu_items (Totem *totem)
 {
-        gboolean playing;
+	GnomeVFSVolume *volume;
+	gboolean playing;
 
 	playing = totem_playing_dvd (totem->mrl);
 
@@ -2621,10 +2637,10 @@ update_media_menu_items (Totem *totem)
 	/* FIXME we should only show that if we have multiple angles */
 	totem_action_set_sensitivity ("next-angle", playing);
 
-	/* FIXME, this really doesn't work right
-	 * since we remove the mediadev property
-	playing = totem_is_media (totem->mrl); */
-	totem_action_set_sensitivity ("eject", FALSE);
+	volume = totem_get_volume_for_media (totem->mrl);
+	totem_action_set_sensitivity ("eject", volume != NULL);
+	if (volume != NULL)
+		gnome_vfs_volume_unref (volume);
 }
 
 static void
@@ -2665,7 +2681,7 @@ main_pane_size_allocated (GtkWidget *main_pane, GtkAllocation *allocation, Totem
 	}
 }
 
-static void
+static char *
 totem_setup_window (Totem *totem)
 {
 	GKeyFile *keyfile;
@@ -2751,7 +2767,7 @@ totem_setup_window (Totem *totem)
 		gtk_widget_modify_bg (vbox, i, &black);
 
 	totem_sidebar_setup (totem, show_sidebar, page_id);
-        g_free (page_id);
+	return page_id;
 }
 
 static void
@@ -2759,6 +2775,7 @@ totem_callback_connect (Totem *totem)
 {
 	GtkWidget *item, *arrow;
 	GtkAction *action;
+	GtkActionGroup *action_group;
 	GtkBox *box;
 
 	/* Menu items */
@@ -2775,14 +2792,6 @@ totem_callback_connect (Totem *totem)
 	/* Controls */
 	box = GTK_BOX (gtk_builder_get_object (totem->xml, "tmw_buttons_hbox"));
 
-	action = gtk_action_group_get_action (totem->main_action_group, "play");
-	item = gtk_action_create_tool_item (action);
-	atk_object_set_name (gtk_widget_get_accessible (item),
-			_("Play / Pause"));
-	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (item),
- 					_("Play / Pause"));
-	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
-
 	action = gtk_action_group_get_action (totem->main_action_group,
 			"previous-chapter");
 	item = gtk_action_create_tool_item (action);
@@ -2792,6 +2801,14 @@ totem_callback_connect (Totem *totem)
 			_("Previous Chapter/Movie"));
 	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
 
+	action = gtk_action_group_get_action (totem->main_action_group, "play");
+	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Play / Pause"));
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (item),
+ 					_("Play / Pause"));
+	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
+
 	action = gtk_action_group_get_action (totem->main_action_group,
 			"next-chapter");
 	item = gtk_action_create_tool_item (action);
@@ -2856,8 +2873,18 @@ totem_callback_connect (Totem *totem)
 	totem_action_set_sensitivity ("play", FALSE);
 	totem_action_set_sensitivity ("next-chapter", FALSE);
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
-	totem_action_set_sensitivity ("skip-forward", FALSE);
-	totem_action_set_sensitivity ("skip-backwards", FALSE);
+	/* FIXME: We can use this code again once bug #457631 is fixed
+	 * and skip-* are back in the main action group. */
+	/*totem_action_set_sensitivity ("skip-forward", FALSE);
+	totem_action_set_sensitivity ("skip-backwards", FALSE);*/
+
+	action_group = GTK_ACTION_GROUP (gtk_builder_get_object (totem->xml, "skip-action-group"));
+
+	action = gtk_action_group_get_action (action_group, "skip-forward");
+	gtk_action_set_sensitive (action, FALSE);
+
+	action = gtk_action_group_get_action (action_group, "skip-backwards");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
@@ -2871,23 +2898,27 @@ playlist_widget_setup (Totem *totem)
 	gtk_widget_show_all (GTK_WIDGET (totem->playlist));
 
 	g_signal_connect (G_OBJECT (totem->playlist), "active-name-changed",
-			G_CALLBACK (on_playlist_change_name), totem);
+			  G_CALLBACK (on_playlist_change_name), totem);
 	g_signal_connect (G_OBJECT (totem->playlist), "item-activated",
-			G_CALLBACK (item_activated_cb), totem);
+			  G_CALLBACK (item_activated_cb), totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"changed", G_CALLBACK (playlist_changed_cb),
-			totem);
+			  "changed", G_CALLBACK (playlist_changed_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"current-removed", G_CALLBACK (current_removed_cb),
-			totem);
+			  "current-removed", G_CALLBACK (current_removed_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"repeat-toggled",
-			G_CALLBACK (playlist_repeat_toggle_cb),
-			totem);
+			  "repeat-toggled",
+			  G_CALLBACK (playlist_repeat_toggle_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"shuffle-toggled",
-			G_CALLBACK (playlist_shuffle_toggle_cb),
-			totem);
+			  "shuffle-toggled",
+			  G_CALLBACK (playlist_shuffle_toggle_cb),
+			  totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			  "subtitle-changed",
+			  G_CALLBACK (subtitle_changed_cb),
+			  totem);
 }
 
 static void
@@ -2897,7 +2928,7 @@ video_widget_create (Totem *totem)
 	GtkContainer *container;
 	BaconVideoWidget **bvw;
 	const GtkTargetEntry source_table[] = {
-		{ "text/uri-list", 0, 0 },
+		{ "text/uri-list", 0, 0 }
 	};
 
 	totem->bvw = BACON_VIDEO_WIDGET
@@ -3031,6 +3062,7 @@ main (int argc, char **argv)
 #endif
 	GOptionContext *context;
 	GOptionGroup *baconoptiongroup;
+	char *sidebar_pageid;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -3085,14 +3117,18 @@ main (int argc, char **argv)
 #endif /* !HAVE_GTK_ONLY */
 
 	totem = g_object_new (TOTEM_TYPE_OBJECT, NULL);
+	totem->gc = gc;
 
 	/* IPC stuff */
-	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
-	totem->gc = gc;
-	if (bacon_message_connection_get_is_server (totem->conn) == FALSE) {
-		totem_options_process_for_server (totem->conn, &optionstate);
-		gdk_notify_startup_complete ();
-		totem_action_exit (totem);
+	if (optionstate.notconnectexistingsession == FALSE) {
+		totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
+		if (bacon_message_connection_get_is_server (totem->conn) == FALSE) {
+			totem_options_process_for_server (totem->conn, &optionstate);
+			gdk_notify_startup_complete ();
+			totem_action_exit (totem);
+		} else {
+			totem_options_process_early (totem, &optionstate);
+		}
 	} else {
 		totem_options_process_early (totem, &optionstate);
 	}
@@ -3128,7 +3164,7 @@ main (int argc, char **argv)
 	totem_setup_file_filters ();
 	totem_setup_play_disc (totem);
 	totem_callback_connect (totem);
-	totem_setup_window (totem);
+	sidebar_pageid = totem_setup_window (totem);
 
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
@@ -3161,17 +3197,18 @@ main (int argc, char **argv)
 	/* Command-line handling */
 	totem_options_process_late (totem, &optionstate);
 
-	/* Initialise all the plugins */
+	/* Initialise all the plugins, and set the default page, in case
+	 * it comes from a plugin */
 	totem_object_plugins_init (totem);
-	//FIXME we should set the current page again, in case a plugin was
-	// the default page
+	totem_sidebar_set_current_page (totem, sidebar_pageid);
+	g_free (sidebar_pageid);
 
 	if (totem->session_restored != FALSE) {
 		totem_session_restore (totem, optionstate.filenames);
 	} else if (optionstate.filenames != NULL && totem_action_open_files (totem, optionstate.filenames)) {
 		totem_action_play_pause (totem);
 	} else {
-		totem_action_set_mrl (totem, NULL);
+		totem_action_set_mrl (totem, NULL, NULL);
 	}
 
 	/* Set the logo at the last minute so we won't try to show it before a video */
@@ -3180,7 +3217,7 @@ main (int argc, char **argv)
 	if (optionstate.fullscreen == FALSE)
 		gdk_window_set_cursor (totem->win->window, NULL);
 
-	if (bacon_message_connection_get_is_server (totem->conn) != FALSE)
+	if (totem->conn != NULL && bacon_message_connection_get_is_server (totem->conn) != FALSE)
 	{
 		bacon_message_connection_set_callback (totem->conn,
 				(BaconMessageReceivedFunc)
